<?php

require __DIR__ . '/CryptAES.php';


/**
 * 辅助函数，用于发送 API 请求
 * @param string $api: 接口地址，例如 /refreshToken
 * @param array $params: 接口参数
 */

function post($api, $params) {
    $post_data = json_encode($params);
    $headers = array('Content-Type: application/json', 'Content-Length: ' . strlen($post_data));
    $ch = curl_init('https://ptp-api.upyun.com' . $api);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $post_data);
    $result = curl_exec($ch);
    if ($result) {
        return json_decode($result, true);
    } else {
        // 客户可以自定义是否使用异常处理错误，可以修改成返回 false 等方式
        throw new Exception('Curl Error: ' . curl_error($ch));
    }
}
/**
 * 辅助函数 AES 加密手机号
 * @param string|int $mobile: 手机号
 * @param string $token: 客户 token
 * @param string $app_key: 客户鉴权账号
 */
function encodeMobile($mobile, $token, $app_key) {
    $aes = new CryptAES();
    $aes->set_key($token);
    $aes->set_iv($app_key);
    $aes->require_pkcs5();

    return $aes->encrypt($mobile);
}

/**
 * 又拍云手机流量营销 SDK
 */
class UpyunMobileStats {

    /**
     * APPKEY 鉴权账号，可以登录又拍云手机流量后台查看
     */
    private $APP_KEY;

    /**
     * APPSECRET 鉴权秘钥，可以登录又拍云手机流量后台查看
     */
    private $APP_SECRET;

    private $TOKEN;

    public function __construct($APP_KEY, $APP_SECRET) {
        $this->APP_KEY = $APP_KEY;
        $this->APP_SECRET = $APP_SECRET;
    }

    /**
     * 获取 TOKEN，TOKEN 有效期为 24 小时
     * 只有最近一次获取的 TOKEN 有效
     */
    function getToken() {
        $result = post('/refreshToken', array(
            'appkey' => $this->APP_KEY,
            'appsecret' => $this->APP_SECRET
        ));
        return $result['token'];
    }

    function setToken($token) {
        $this->TOKEN = $token; 
    }

    /**
     * 创建充值订单
     * @param string $prodcode: 又拍云手机流量包产品编码
     * @param string|int $custno: 客户自定义订单号
     * @param string|int $mobile: 申请流量包的手机号
     */
    function createOrder($prodcode, $custno, $mobile) {
        $mobile_aes = encodeMobile($mobile, $this->TOKEN, $this->APP_KEY);
        $params = array(
            'appkey'  => $this->APP_KEY,
            'mobile' => $mobile_aes,
            'custno' => $custno,
            'prodcode' => $prodcode,
        );
        // 按照参数 key，进行字典序正序拼接，最后再拼接 token
        ksort($params);
        $str = '';
        foreach($params as $k => $v) {
            $str .= $k.$v; 
        }
        $str .= "token" . $this->TOKEN;
        $sign = sha1($str);

        $params['sign'] = $sign;
        $result = post('/chargeOrder', $params);
        return $result;
    }

    /**
     * 查询订单状态
     * @param string|int $custno: 订单号
     * @param string $request_time: 查询的时间，传递该值时，表示该时间往后一小时内查询，不传递表示最近一周内查询
     */
    function queryOrder($custno, $request_time) {
        $str = "appkey" . $this->APP_KEY . "custno${custno}";
        if ($request_time) {
            $str .= "requesttime${request_time}";
        }
        $str .= "TOKEN" . $this->TOKEN;

        $result = post('/seekOrder', array(
            'appkey' => $this->APP_KEY,
            'custno' => $custno,
            'requesttime' => $request_time,
            'sign' => sha1($str),
        ));
        return $result;
    }

    /**
     * 查询账户余额
     */
    function queryBalance() {
        $str = "appkey" . $this->APP_KEY;
        $str .= "TOKEN" . $this->TOKEN;
        $result = post('/getMyBalance', array(
            'appkey' => $this->APP_KEY,
            'sign' => sha1($str),
        ));
        return $result;
    }
}
